#ifndef SLICESAX2HANDLER_HH
#define SLICESAX2HANDLER_HH 1

#include <xercesc/sax2/Attributes.hpp>
#include <xercesc/sax2/DefaultHandler.hpp>
#include <xercesc/sax/SAXParseException.hpp>
#include <xercesc/sax/SAXException.hpp>

#include <xercesc/sax2/XMLReaderFactory.hpp>
#include <xercesc/sax2/SAX2XMLReader.hpp>
#include <xercesc/framework/StdInInputSource.hpp>
#include <xercesc/util/PlatformUtils.hpp>

#include <Foundation.hh>
#include "ConsensusRequest.hh"
#include "StrX.hh"

#include "SliceData.hh"

using libSlice::SliceData;
using libSlice::ReadData;
using libSlice::ReadManager;

using namespace std;

class RequestManager;
#include "RequestManager.hh"


//! Xerces SAX2 Interface
class SliceSAX2Handler : public DefaultHandler
{
public:
  SliceSAX2Handler(RequestManager * manager,
                   LogCategory & log,
                   bool doLogging = true);

  ~SliceSAX2Handler();

  void printStats(ostream & out, const string & infile);

  // -----------------------------------------------------------------------
  //  Handlers for the SAX ContentHandler interface
  // -----------------------------------------------------------------------
  void startElement(const XMLCh* const uri, 
                    const XMLCh* const localname, 
                    const XMLCh* const qname, 
                    const Attributes& attrs);

  void characters(const XMLCh* const chars, 
                  const unsigned int length);

  void endDocument();


  // -----------------------------------------------------------------------
  //  Handlers for the SAX ErrorHandler interface
  // -----------------------------------------------------------------------
	void warning    (const SAXParseException& exception);
  void error      (const SAXParseException& exception);
  void fatalError (const SAXParseException& exception);


private:

  enum XMLLocation
  {
    IN_UNKNOWN,
    IN_NUC,
    IN_QUALVAL,
    IN_READID,

    IN_LEFT,
    IN_LEFT_NUC,
    IN_LEFT_QUAL,
    IN_RIGHT,
    IN_RIGHT_NUC,
    IN_RIGHT_QUAL
  };

  XMLLocation m_location; 

  // -----------------------------------------------------------------------
  //  Handlers for coverting to slice objects
  // -----------------------------------------------------------------------
  
  void handleRequestData(const char * type, 
                         const char * host, 
                         const char * user, 
                         const char * id,
                         const char * option);

  void handleSimpleOption(const char * name, 
                          const char * value);

  void handleReadData   (const char * id, 
                         const char * seqname,
                         const char * dir, 
                         const char * chem);

  void handleClrData    (const char * left,
                         const char * right);

  void handleSliceRange (const char * index, 
                         const char * gindex);

  void handleSliceData  (const char * index, 
                         const char * gindex,
                         const char * existing);

  void handleNuc        (const char * nucData);
  void handleReadId     (const char * readId);
  void handleQualVal    (const char * qualVal);

  void handleTrimmedSeq (const char * position);
  void handleTrimmedQual(const char * buffer, XMLLocation location);
  void handleTrimmedNuc (const char * buffer, XMLLocation location);

  void endCurrentSlice();

  // -----------------------------------------------------------------------
  //  Private Data
  // -----------------------------------------------------------------------

  unsigned long m_startMillis;

  unsigned int m_attrCount;
  unsigned int m_characterCount;
  unsigned int m_elementCount;

  unsigned int m_sliceRangeCount;
  unsigned int m_sliceCount;
  unsigned int m_readCount;

  long m_sliceRangeIndex;
  long m_sliceRangeGindex;

  ReadManager m_readManager;
  ReadData * m_currentRead;

  SliceData * m_currentSlice;

  RequestManager * m_manager;

  LogCategory & m_log;
  bool m_doLogging;

};

#endif
