#include "SliceReverser.hh"
#include <cstdio>

SliceReverser::SliceReverser() : RequestManager("SliceReverser")
{
  m_log.debugL("Contructed SliceReverser");
  m_log.setMessageLevel(MESSAGE_LEVEL_DEBUG_MEDIUM);
  
  m_index = -1;
  m_gindex = -1;
}

SliceReverser::~SliceReverser()
{
  m_log.debugL("Destructed SliceReverser");
}

void SliceReverser::addSliceRange(long & index, long & gindex)
{
  m_out << "  </ReadCollection>" << endl;

  m_out << "  <SliceRange Index=\"" << index
        << "\" Gindex=\""           << gindex
        << "\">" << endl;

  index = gindex = 0L;
}

void SliceReverser::addSlice(SliceData * slice)
{
  RequestManager::addSlice(slice);

  if (m_index == -1)
  {
    // This is the first slice
    m_index = slice->getIndex();
    m_gindex = slice->getGindex();
  }

  // Add the slice to the queue
  m_sliceList.push_back(slice);
}

void SliceReverser::addReadData(ReadData * readData)
{
  // Swap the direction and the clear range
  readData->reverseCompliment();
  
  // Now print
  readData->printXML(m_out);
}

void SliceReverser::addRequest(ConsensusRequest * request)
{
  m_requestData = request;

  printXMLHeader(request);
  m_out << "  <ReadCollection>" << endl;
}

void SliceReverser::printReverseSliceXML(SliceData * slice)
{
  char consensus = slice->getConsensus();

  long coordinate = m_index;

  if (consensus == '-' && coordinate) coordinate--;

  m_out << "    <Slice Index=\"" << coordinate
        << "\" Gindex=\""        << m_gindex
        << "\" Existing=\""      << libSlice_getCompliment(consensus)
        << "\">" << endl;

  m_out << "      <Nuc>";

  unsigned int depth = slice->getDepth();
  for (unsigned int i=0; i < depth; i++)
  {
    m_out << libSlice_getCompliment(slice->getBase(i));
  }

  m_out << "</Nuc>" << endl;

  m_out << "      <Qualval>" 
        << slice->getQualValData() 
        << "</Qualval>" 
        << endl;

  m_out << "      <ReadID>"  
        << slice->getReadIdData() 
        << "</ReadID>"  
        << endl;

  m_out << "    </Slice>" << endl;

  if (consensus != '-') m_index++;
  m_gindex++;
}

int SliceReverser::printResults()
{
  // No more slices, walk through slice list backwards, taking complement
  // along the way

  if (!m_errorCount)
  {
    SliceList::reverse_iterator i;
    for (i =  m_sliceList.rbegin();
         i != m_sliceList.rend();
         i++)
    {
      printReverseSliceXML(*i);
      delete *i;
    }
  }

  m_out << "  </SliceRange>" << endl;
  m_out << "</Request>"      << endl;

  return m_errorCount;
}
