#include "SliceRotator.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = 
"Rotates the tiling effectively moving the beginning of the consensus to the\n"
"end. \n"
"\n"
"   Usage: rotateSlice [options] slice.xml\n"
"\n"
"     slice.xml Slice XML file\n"
"\n"
"   rotateSlice Options\n"
"   ------------------\n"
"   -n <count> Specify number of slices to rotate by\n"
"   -L         Assume contig is linear, renumber rotated reads with read'\n"
 + RequestManager::getOptionHelp();

    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    SliceRotator::handleOptions(tf);
    SliceRotator manager;

    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an input file", "rotateSlice");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
