#include "TilingManager.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = 
"Converts a Slice XML file into contig format.\n"
"\n"
"   Usage: slice2contig [options] slice.xml\n"
"\n"
"     slice.xml Slice XML file\n"
"\n"
"   slice2contig Options\n"
"   --------------------\n"
"   -i|--asmbl_id <id> Specify the id for the contig\n"
"   -C                 Allow for circular contigs\n"
"                        (Some reads may have negative coordinates)\n"
 + RequestManager::getOptionHelp();


    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    TilingManager::handleOptions(tf);
    TilingManager manager;

    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an input file", "slice2contig");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
