#include "ReadStripper.hh"
#include "version.h"

int main(int argc, char ** argv)
{
  int retval = 0;
  string infile;

  Foundation * tf = NULL;

  try
  {
    string version = RequestManager::getVersionString(gVersionStr,
                                                      gRevisionStr);
    string dependencies = "xerces";
    string helptext = 
"Strip one more read reads from the tiling in Slice XML format.\n"
"\n"
"   Usage: stripSlice [options] slice.xml\n"
"\n"
"     slice.xml Slice XML file\n"
"\n"
"   stripSlice Options\n"
"   ------------------\n"
"   -i|--id <id> Specify single read id to strip\n"
"   -I|--idlist <file> Specify file of read ids to strip\n"
"\n"
"   -n|--seqname <name> Specify single seqname to strip\n"
"   -N|--seqlist <file> Specify file of seqnames to strip\n"
"\n"
"   -r|--renumber Renumber read ids after stripping\n"
 + RequestManager::getOptionHelp();

    // Instantiate a new Foundation object
    tf = new Foundation (version, helptext, dependencies, argc, argv);
    tf->disableOptionHelp();

    ReadStripper::handleOptions(tf);
    ReadStripper manager;

    // Read the input filename
    if ((infile = tf->getOptions()->getNextOtherData()) == "")
    {
      throw Exception("You must specify an input file", "stripSlice");
    }

    // Process the file
    retval = manager.processFile(infile, cerr);
  }
  catch (const ExitProgramNormally & e)
  {
    retval = 0;
  }
  catch (const Exception & e)
  {
    cerr << e << endl;
    retval = 100;
  }

  try
  {
    if (tf) delete tf;
  }
  catch (const Exception & e)
  {
    cerr << "Exception while deleting tf: " << e << endl;
    retval = 105;
  }

  return retval;
}
