#ifndef READDATA_HH
#define READDATA_HH 1

/*! @file
 *  @brief Defines interface for ReadData and ReadManager
 */

#include <string>
#include <map>
#include <map>
#include <vector>

using namespace std;

/*
template<> struct hash< std::string >
{
  size_t operator()( const std::string& x ) const
  {
          return hash< const char* >()( x.c_str() );
  }
};
*/

namespace libSlice
{
  using std::string;
  using std::map;
  using std::vector;

  /*! @brief Maintains information about a given read
   *  @ingroup cppinterface
   */
  class ReadData
  {
  public:
    ReadData(const char * id, 
             const char * seqname,
             const char * dir, 
             const char * chem);

    void addClrData (const char * left,
                     const char * right);

    void setClrData(long int clrleft, long int clrright);
    void setSeqData(long int seqleft, long int seqright);

    void addNucData (const char * buffer, bool isLeft);
    void addQualData(const char * buffer, bool isLeft);

    void flushTrimmedSeq();
    void flushTrimmedSeqSide(bool flushLeft);

    void reverseCompliment();

    void resetId(const string & id);

    char           isReverseCompliment()                  const;

    void           printXML(ostream & out, 
                            const string & prefix="    ") const;

    long           getSeqleft()                           const;
    long           getSeqright()                          const;
    long           getClrleft()                           const;
    long           getClrright()                          const;

    const string & getId()                                const;
    const string & getSeqname()                           const;
    const string & getChem()                              const;
    const string & getRawNucData(bool getLeft)            const;
    const string & getRawQualData(bool getLeft)           const;

    const string   getNucData(bool getLeft)               const;
    const string   getQualData(bool getLeft)              const;

  private:
    void validateSeqdata()                                const;
    void parseDirection();

    string m_id;
    string m_seqname;
    string m_dir;
    string m_chem;

    string m_leftnuc;
    string m_leftqual;
    string m_rightnuc;
    string m_rightqual;

    long m_seqleft;
    long m_seqright;
    long m_clrleft;
    long m_clrright;

    char m_rc;
  };

  //! Alias for referencing map datatype
  typedef map<string, ReadData *> ReadMap;

  /*! 
   *  @ingroup cppinterface
   *  @brief Manages the information about all of the read data
   */
  class ReadManager
  {
  public:
    ReadManager();
    ~ReadManager();

    int size() const;

    ReadData * addReadData(const char * id, 
                           const char * seqname,
                           const char * dir, 
                           const char * chem);

    void       insertRead(ReadData * rd);

    void       resetReadId(ReadData * rd, const string & id);
    void       removeRead(const string & id);

    ReadData * getNextRead();
    void       resetReadIterator();

    void       printXML(ostream & out, const string & prefix="  ") const;

    ReadData * find(const string & readId)                         const;

  private:

    ReadMap m_readMap;
    ReadMap::const_iterator m_readIterator;
  };
}

#endif
